package
{
	import away3d.animators.ParticleAnimationSet;
	import away3d.animators.ParticleAnimator;
	import away3d.animators.data.ParticleProperties;
	import away3d.animators.data.ParticlePropertiesMode;
	import away3d.animators.nodes.ParticleBillboardNode;
	import away3d.animators.nodes.ParticleColorNode;
	import away3d.animators.nodes.ParticleFollowNode;
	import away3d.animators.nodes.ParticleVelocityNode;
	import away3d.containers.ObjectContainer3D;
	import away3d.containers.View3D;
	import away3d.core.base.Geometry;
	import away3d.core.base.Object3D;
	import away3d.core.base.ParticleGeometry;
	import away3d.debug.AwayStats;
	import away3d.entities.Mesh;
	import away3d.materials.TextureMaterial;
	import away3d.primitives.PlaneGeometry;
	import away3d.primitives.WireframePlane;
	import away3d.tools.helpers.ParticleGeometryHelper;
	import away3d.tools.helpers.data.ParticleGeometryTransform;
	import away3d.utils.Cast;

	import flash.display.Bitmap;
	import flash.display.BlendMode;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.geom.ColorTransform;
	import flash.geom.Vector3D;
	import flash.utils.getTimer;

	import frocessing.color.ColorHSV;

	[SWF(frameRate = '60', width = "960", height = "540")]
	public class Main extends View3D
	{

		//----------------------------------------------------------
		//
		//   Constructor 
		//
		//----------------------------------------------------------

		public function Main()
		{
			if (stage)
				initialize(null);
			else
				addEventListener(Event.ADDED_TO_STAGE, initialize);
		}

		//----------------------------------------------------------
		//
		//   Property 
		//
		//----------------------------------------------------------

		[Embed(source = "assets/bg.png")]
		private var BgImg:Class;

		[Embed(source = "assets/cloud_1.png")]
		private var ParticleImg1:Class;
		[Embed(source = "assets/cloud_2.png")]
		private var ParticleImg2:Class;
		[Embed(source = "assets/cloud_3.png")]
		private var ParticleImg3:Class;
		[Embed(source = "assets/cloud_4.png")]
		private var ParticleImg4:Class;
		[Embed(source = "assets/star.png")]
		private var ParticleImg5:Class;
		private var followTarget:ObjectContainer3D;
		private var particleFollowNode:ParticleFollowNode;
		private var particleGeometry:ParticleGeometry;

		//----------------------------------------------------------
		//
		//   Function 
		//
		//----------------------------------------------------------

		private function initialize(event:Event):void
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			antiAlias = 4;
			background = Cast.bitmapTexture(new BgImg());
			addChild(new AwayStats(this));

			// Floor
			var floor:WireframePlane = new WireframePlane(2000, 2000, 40, 40, 0x999999, 1, WireframePlane.ORIENTATION_XZ);
			floor.y = -450;
			scene.addChild(floor);

			// Particles
			initParticles();
			followTarget = new ObjectContainer3D();
			scene.addChild(followTarget);

			createParticle(followTarget, new ParticleImg1(), 290); // スモーク風パーティクル
			createParticle(followTarget, new ParticleImg2(), 180); // スモーク風パーティクル
			createParticle(followTarget, new ParticleImg3(), 220); // スモーク風パーティクル
			createParticle(followTarget, new ParticleImg4(), 270); // スモーク風パーティクル
			createParticle(followTarget, new ParticleImg5(), 300); // 星型パーティクル

			// Start
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
			stage.addEventListener(Event.RESIZE, resizeHandler);
		}

		/**
		 * EnterFrame
		 */
		private function enterFrameHandler(ev:Event):void
		{
			// Update Camera
			var msec:Number = getTimer();
			camera.x = 1500 * Math.sin(msec / 1200);
			camera.z = 1500 * Math.cos(msec / 1200);
			camera.y = 500 * Math.sin(msec / 3000) + 100;
			camera.lookAt(new Vector3D());

			// Update Particles
			var time:Number = getTimer();
			var rot:Number = time / 3;
			var distance:Number = (time * 0.25) % 2000 - 1000;
			// run circle
			followTarget.x = 300 * Math.cos(-rot * Math.PI / 180);
			followTarget.y = 300 * Math.sin(-rot * Math.PI / 180);
			followTarget.z = distance;

			// Render
			render();
		}

		/**
		 * Initialise the particles
		 */
		private function initParticles():void
		{
			//setup the base geometry for one particle
			var plane:Geometry = new PlaneGeometry(60, 60, 1, 1, false);

			//create the particle geometry
			var geometrySet:Vector.<Geometry> = new Vector.<Geometry>();
			var setTransforms:Vector.<ParticleGeometryTransform> = new Vector.<ParticleGeometryTransform>();
			var particleTransform:ParticleGeometryTransform;
			for (var i:int = 0; i < 1000; i++)
			{
				geometrySet.push(plane);
				particleTransform = new ParticleGeometryTransform();
				setTransforms.push(particleTransform);
			}

			particleGeometry = ParticleGeometryHelper.generateGeometry(geometrySet, setTransforms);
		}

		/**
		 * Initialiser function for particle properties
		 */
		private function initParticleProperties(properties:ParticleProperties):void
		{
			properties.startTime = Math.random() * 4.1;
			properties.duration = 2 + 2 * Math.random(); // 継続時間
			// 移動量
			properties[ParticleVelocityNode.VELOCITY_VECTOR3D] = new Vector3D(
				(Math.random() - 0.5) * 200,
				(Math.random() - 0.5) * 200,
				(Math.random() - 0.5) * 200 - 200);
		}

		/**
		 * Create Particles 
		 * @param followTarget	追随させたいターゲット
		 * @param bitmap	ビットマップオブジェクト
		 * @param hue	色相(0〜360)
		 * 
		 */
		private function createParticle(followTarget:Object3D, bitmap:Bitmap, hue:Number):void
		{
			var particleMaterial:TextureMaterial = new TextureMaterial(Cast.bitmapTexture(bitmap));
			particleMaterial.alphaBlending = true;
			particleMaterial.blendMode = BlendMode.ADD;

			//create the particle animation set
			var particleAnimationSet:ParticleAnimationSet = new ParticleAnimationSet(true, true, true);

			// set color hsv
			var hsv:ColorHSV = new ColorHSV(hue, 0.6, 1.0);
			var colorR:Number = hsv.r / 255;
			var colorG:Number = hsv.g / 255;
			var colorB:Number = hsv.b / 255;

			//define the particle animations and init function
			particleAnimationSet.addAnimation(new ParticleBillboardNode());
			particleAnimationSet.addAnimation(new ParticleVelocityNode(ParticlePropertiesMode.LOCAL_STATIC));
			particleAnimationSet.addAnimation(new ParticleColorNode(ParticlePropertiesMode.GLOBAL, true, true, false, false, new ColorTransform(colorR, colorG, colorB, 1), new ColorTransform(colorR, colorG, colorB, 0.25)));
			particleAnimationSet.addAnimation(particleFollowNode = new ParticleFollowNode(true, false));
			particleAnimationSet.initParticleFunc = initParticleProperties;

			//create the particle meshes
			var particleMesh:Mesh = new Mesh(particleGeometry, particleMaterial);
			scene.addChild(particleMesh);

			//create and start the particle animators
			var animator:ParticleAnimator = new ParticleAnimator(particleAnimationSet);
			particleMesh.animator = animator;
			animator.start();
			particleFollowNode.getAnimationState(animator).followTarget = followTarget;
		}

		/**
		 * Resize
		 */
		private function resizeHandler(event:Event):void
		{
			this.width = stage.stageWidth;
			this.height = stage.stageHeight;
		}
	}
}

